#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QBrush>
#include <unistd.h>
#include <QtConcurrent/QtConcurrentRun>
#include <iostream>
using namespace std;

#define PI_LOCATION_X      50
#define PI_TEXT_LOCATION_Y 50
#define PI_RECT_LOCATION_Y 60
#define PI_RECT_WIDTH      50
#define PI_RECT_HEIGHT     10

MainWindow::MainWindow(QWidget *parent) :
  QMainWindow(parent),
  ui(new Ui::MainWindow)
{
  ui->setupUi(this);
  this->setGeometry(0, 0, SIZE, SIZE);
  srand(time(NULL));
  QtConcurrent::run(this, &MainWindow::generatePoints);
}

MainWindow::~MainWindow()
{
  delete ui;
}

//Pre:  None
//Post: Draws the current state of the points array to the current window.
void MainWindow::paintEvent(QPaintEvent *) {
  QPainter painter(this);

  painter.drawEllipse(0, 0, SIZE, SIZE);

  for(int i = 0; i < currNumPoints; i++) {
    Point p = points[i];
    QColor c = Qt::red;

    if(inCircle(p)) {
      c = Qt::green;
    }

    int x = p.getX();
    int y = p.getY();

    painter.setBrush(QBrush(c));
    painter.drawEllipse(x - 1, y - 1, 2, 2);
  }

  painter.fillRect(SIZE - PI_LOCATION_X, SIZE - PI_RECT_LOCATION_Y, PI_RECT_WIDTH, PI_RECT_HEIGHT, Qt::white);
  painter.setBrush(QBrush(Qt::black));
  painter.drawText(SIZE - PI_LOCATION_X, SIZE - PI_TEXT_LOCATION_Y, QString::number(4 * ((float)countIn / currNumPoints)));
}

//Pre:  points is an MAX_POINTS size array
//Post: Incrementally generates new points, and forces an animation update.
void MainWindow::generatePoints() {
  countIn = 0;

  for(currNumPoints = 0; currNumPoints < MAX_POINTS; currNumPoints++) {
    Point p(rand() % SIZE, rand() % SIZE);
    points[currNumPoints] = p;
    if(inCircle(p)) {
      countIn++;
    }
    update();
    usleep(5000);
  }

}

//Pre:  p is a valid point.
//Post: Returns true if p is inside the circle inscribed in this window.
bool MainWindow::inCircle(Point p) {
  int cX = SIZE / 2;;
  int cY = SIZE / 2;;

  int xDist = p.getX() - cX;
  int yDist = p.getY() - cY;

  return sqrt(xDist * xDist + yDist * yDist) < (SIZE / 2);
}
